@@ -47,6 +47,7 @@ class OneLogin_Saml2_Logout_Request(object):
         """
         self.__settings = settings
         self.__error = None
+        self.id = None
 
         if request is None:
             sp_data = self.__settings.get_sp_data()
@@ -54,6 +55,8 @@ class OneLogin_Saml2_Logout_Request(object):
             security = self.__settings.get_security_data()
 
             uid = OneLogin_Saml2_Utils.generate_unique_id()
+            self.id = uid
+
             issue_instant = OneLogin_Saml2_Utils.parse_time_to_SAML(OneLogin_Saml2_Utils.now())
 
             cert = None
@@ -105,6 +108,7 @@ class OneLogin_Saml2_Logout_Request(object):
                 logout_request = inflated
             except Exception:
                 logout_request = decoded
+            self.id = self.get_id(logout_request)
 
         self.__logout_request = logout_request
 
@@ -302,9 +306,6 @@ class OneLogin_Saml2_Logout_Request(object):
                 else:
                     sign_alg = get_data['SigAlg']
 
-                if sign_alg != OneLogin_Saml2_Constants.RSA_SHA1:
-                    raise Exception('Invalid signAlg in the recieved Logout Request')
-
                 signed_query = 'SAMLRequest=%s' % quote_plus(get_data['SAMLRequest'])
                 if 'RelayState' in get_data:
                     signed_query = '%s&RelayState=%s' % (signed_query, quote_plus(get_data['RelayState']))
@@ -314,12 +315,12 @@ class OneLogin_Saml2_Logout_Request(object):
                     raise Exception('In order to validate the sign on the Logout Request, the x509cert of the IdP is required')
                 cert = idp_data['x509cert']
 
-                if not OneLogin_Saml2_Utils.validate_binary_sign(signed_query, b64decode(get_data['Signature']), cert):
+                if not OneLogin_Saml2_Utils.validate_binary_sign(signed_query, b64decode(get_data['Signature']), cert, sign_alg):
                     raise Exception('Signature validation failed. Logout Request rejected')
 
             return True
         except Exception as err:
-            # pylint: disable=R0801
+            # pylint: disable=R0801sign_alg
             self.__error = err.__str__()
             debug = self.__settings.is_debug_active()
             if debug:
