@@ -12,7 +12,7 @@ Auxiliary class of OneLogin's Python Toolkit.
 import base64
 from datetime import datetime
 import calendar
-from hashlib import sha1
+from hashlib import sha1, sha256, sha384, sha512
 from isodate import parse_duration as duration_parser
 from lxml import etree
 from defusedxml.lxml import tostring, fromstring
@@ -522,14 +522,17 @@ class OneLogin_Saml2_Utils(object):
             callback()
 
     @staticmethod
-    def calculate_x509_fingerprint(x509_cert):
+    def calculate_x509_fingerprint(x509_cert, alg='sha1'):
         """
         Calculates the fingerprint of a x509cert.
 
         :param x509_cert: x509 cert
         :type: string
 
-        :returns: Formated fingerprint
+        :param alg: The algorithm to build the fingerprint
+        :type: string
+
+        :returns: fingerprint
         :rtype: string
         """
         assert isinstance(x509_cert, basestring)
@@ -552,9 +555,19 @@ class OneLogin_Saml2_Utils(object):
             else:
                 # Append the current line to the certificate data.
                 data += line
-        # "data" now contains the certificate as a base64-encoded string. The
-        # fingerprint of the certificate is the sha1-hash of the certificate.
-        return sha1(base64.b64decode(data)).hexdigest().lower()
+
+        decoded_data = base64.b64decode(data)
+
+        if alg == 'sha512':
+            fingerprint = sha512(decoded_data)
+        elif alg == 'sha384':
+            fingerprint = sha384(decoded_data)
+        elif alg == 'sha256':
+            fingerprint = sha256(decoded_data)
+        else:
+            fingerprint = sha1(decoded_data)
+
+        return fingerprint.hexdigest().lower()
 
     @staticmethod
     def format_finger_print(fingerprint):
@@ -746,7 +759,7 @@ class OneLogin_Saml2_Utils(object):
         return f_temp
 
     @staticmethod
-    def add_sign(xml, key, cert, debug=False):
+    def add_sign(xml, key, cert, debug=False, sign_algorithm=OneLogin_Saml2_Constants.RSA_SHA1):
         """
         Adds signature key and senders certificate to an element (Message or
         Assertion).
@@ -757,11 +770,14 @@ class OneLogin_Saml2_Utils(object):
         :param key: The private key
         :type: string
 
+        :param cert: The public
+        :type: string
+
         :param debug: Activate the xmlsec debug
         :type: bool
 
-        :param cert: The public
-        :type: string
+        :param sign_algorithm: Signature algorithm method
+        :type sign_algorithm: string
         """
         if xml is None or xml == '':
             raise Exception('Empty string supplied as input')
@@ -793,8 +809,17 @@ class OneLogin_Saml2_Utils(object):
         if debug:
             xmlsec.set_error_callback(print_xmlsec_errors)
 
-        # Sign the metadacta with our private key.
-        signature = Signature(xmlsec.TransformExclC14N, xmlsec.TransformRsaSha1)
+        # Sign the metadata with our private key.
+        sign_algorithm_transform_map = {
+            OneLogin_Saml2_Constants.DSA_SHA1: xmlsec.TransformDsaSha1,
+            OneLogin_Saml2_Constants.RSA_SHA1: xmlsec.TransformRsaSha1,
+            OneLogin_Saml2_Constants.RSA_SHA256: xmlsec.TransformRsaSha256,
+            OneLogin_Saml2_Constants.RSA_SHA384: xmlsec.TransformRsaSha384,
+            OneLogin_Saml2_Constants.RSA_SHA512: xmlsec.TransformRsaSha512
+        }
+        sign_algorithm_transform = sign_algorithm_transform_map.get(sign_algorithm, xmlsec.TransformRsaSha1)
+
+        signature = Signature(xmlsec.TransformExclC14N, sign_algorithm_transform)
 
         issuer = OneLogin_Saml2_Utils.query(elem, '//saml:Issuer')
         if len(issuer) > 0:
@@ -837,7 +862,7 @@ class OneLogin_Saml2_Utils(object):
         return newdoc.saveXML(newdoc.firstChild)
 
     @staticmethod
-    def validate_sign(xml, cert=None, fingerprint=None, validatecert=False, debug=False):
+    def validate_sign(xml, cert=None, fingerprint=None, fingerprintalg='sha1', validatecert=False, debug=False):
         """
         Validates a signature (Message or Assertion).
 
@@ -850,6 +875,9 @@ class OneLogin_Saml2_Utils(object):
         :param fingerprint: The fingerprint of the public cert
         :type: string
 
+        :param fingerprintalg: The algorithm used to build the fingerprint
+        :type: string
+
         :param validatecert: If true, will verify the signature and if the cert is valid.
         :type: bool
 
@@ -899,13 +927,19 @@ class OneLogin_Saml2_Utils(object):
                     if len(x509_certificate_nodes) > 0:
                         x509_certificate_node = x509_certificate_nodes[0]
                         x509_cert_value = x509_certificate_node.text
-                        x509_fingerprint_value = OneLogin_Saml2_Utils.calculate_x509_fingerprint(x509_cert_value)
+                        x509_fingerprint_value = OneLogin_Saml2_Utils.calculate_x509_fingerprint(x509_cert_value, fingerprintalg)
                         if fingerprint == x509_fingerprint_value:
                             cert = OneLogin_Saml2_Utils.format_cert(x509_cert_value)
 
                 if cert is None or cert == '':
                     return False
 
+                # Check if Reference URI is empty
+                reference_elem = OneLogin_Saml2_Utils.query(signature_node, '//ds:Reference')
+                if len(reference_elem) > 0:
+                    if reference_elem[0].get('URI') == '':
+                        reference_elem[0].set('URI', '#%s' % signature_node.getparent().get('ID'))
+
                 dsig_ctx = xmlsec.DSigCtx()
 
                 file_cert = OneLogin_Saml2_Utils.write_temp_file(cert)
@@ -929,7 +963,7 @@ class OneLogin_Saml2_Utils(object):
             return False
 
     @staticmethod
-    def validate_binary_sign(signed_query, signature, cert=None, algorithm=xmlsec.TransformRsaSha1, debug=False):
+    def validate_binary_sign(signed_query, signature, cert=None, algorithm=OneLogin_Saml2_Constants.RSA_SHA1, debug=False):
         """
         Validates signed bynary data (Used to validate GET Signature).
 
@@ -961,7 +995,17 @@ class OneLogin_Saml2_Utils(object):
             dsig_ctx.signKey = xmlsec.Key.load(file_cert.name, xmlsec.KeyDataFormatCertPem, None)
             file_cert.close()
 
-            dsig_ctx.verifyBinary(signed_query, algorithm, signature)
+            # Sign the metadata with our private key.
+            sign_algorithm_transform_map = {
+                OneLogin_Saml2_Constants.DSA_SHA1: xmlsec.TransformDsaSha1,
+                OneLogin_Saml2_Constants.RSA_SHA1: xmlsec.TransformRsaSha1,
+                OneLogin_Saml2_Constants.RSA_SHA256: xmlsec.TransformRsaSha256,
+                OneLogin_Saml2_Constants.RSA_SHA384: xmlsec.TransformRsaSha384,
+                OneLogin_Saml2_Constants.RSA_SHA512: xmlsec.TransformRsaSha512
+            }
+            sign_algorithm_transform = sign_algorithm_transform_map.get(algorithm, xmlsec.TransformRsaSha1)
+
+            dsig_ctx.verifyBinary(signed_query, sign_algorithm_transform, signature)
             return True
         except Exception:
             return False
